@@ -1,14 +1,15 @@
 package Maypole::Model::CDBI::Plain;
 use base 'Maypole::Model::CDBI';
+
 sub setup_database {
-    my ($self, $config, $namespace, $classes) = @_;
-    $config->{classes} = $classes;
-    $config->{table_to_class}  = { map { $_->table => $_ } @$classes };
-    $config->{tables} = [ keys %{$config->{table_to_class}} ];
+    my ( $self, $config, $namespace, $classes ) = @_;
+    $config->{classes}        = $classes;
+    $config->{table_to_class} = { map { $_->table => $_ } @$classes };
+    $config->{tables}         = [ keys %{ $config->{table_to_class} } ];
 }
 
 sub class_of {
-    my ($self, $r, $table) = @_;
+    my ( $self, $r, $table ) = @_;
     return $r->config->{table_to_class}->{$table};
 }
 
@@ -21,7 +22,7 @@ Maypole::Model::CDBI::Plain - Class::DBI model without ::Loader
 =head1 SYNOPSIS
 
     package Foo;
-    use base 'Apache::DBI';
+    use base 'Maypole::Application';
     use Foo::SomeTable;
     use Foo::Other::Table;
 
@@ -35,4 +36,17 @@ C<Class::DBI> classes; simply call C<setup> with a list reference
 of the classes you're going to use, and Maypole will work out the
 tables and set up the inheritance relationships as normal.
 
+=head1 METHODS
+
+=over 4
+
+=item setup_database
+
+=item  class_of
+
+=back
+
+See L<Maypole::Model::Base>
+
 =cut
+
